@@ -28,10 +28,11 @@ class BUFFER_STATE : public BINDABLE {
   public:
     const safe_VkBufferCreateInfo safe_create_info;
     const VkBufferCreateInfo &createInfo;
-    VkDeviceAddress deviceAddress;
     const VkMemoryRequirements requirements;
     const VkMemoryRequirements *const memory_requirements_pointer = &requirements;
-    bool memory_requirements_checked;
+    VkDeviceAddress deviceAddress = 0;
+    // VkBufferUsageFlags2CreateInfoKHR can be used instead over the VkBufferCreateInfo::usage
+    const VkBufferUsageFlags2KHR usage;
 
     vvl::unordered_set<std::shared_ptr<const VideoProfileDesc>> supported_video_profiles;
 
@@ -59,21 +60,6 @@ using BUFFER_STATE_LINEAR = MEMORY_TRACKED_RESOURCE_STATE<BUFFER_STATE, Bindable
 template <bool IS_RESIDENT>
 using BUFFER_STATE_SPARSE = MEMORY_TRACKED_RESOURCE_STATE<BUFFER_STATE, BindableSparseMemoryTracker<IS_RESIDENT>>;
 
-#ifdef VK_USE_PLATFORM_METAL_EXT
-static bool GetMetalExport(const VkBufferViewCreateInfo *info) {
-    bool retval = false;
-    auto export_metal_object_info = LvlFindInChain<VkExportMetalObjectCreateInfoEXT>(info->pNext);
-    while (export_metal_object_info) {
-        if (export_metal_object_info->exportObjectType == VK_EXPORT_METAL_OBJECT_TYPE_METAL_TEXTURE_BIT_EXT) {
-            retval = true;
-            break;
-        }
-        export_metal_object_info = LvlFindInChain<VkExportMetalObjectCreateInfoEXT>(export_metal_object_info->pNext);
-    }
-    return retval;
-}
-#endif
-
 class BUFFER_VIEW_STATE : public BASE_NODE {
   public:
     const VkBufferViewCreateInfo create_info;
@@ -86,15 +72,7 @@ class BUFFER_VIEW_STATE : public BASE_NODE {
     const VkFormatFeatureFlags2KHR buf_format_features;
 
     BUFFER_VIEW_STATE(const std::shared_ptr<BUFFER_STATE> &bf, VkBufferView bv, const VkBufferViewCreateInfo *ci,
-                      VkFormatFeatureFlags2KHR buf_ff)
-        : BASE_NODE(bv, kVulkanObjectTypeBufferView),
-          create_info(*ci),
-          buffer_state(bf),
-#ifdef VK_USE_PLATFORM_METAL_EXT
-          metal_bufferview_export(GetMetalExport(ci)),
-#endif
-          buf_format_features(buf_ff) {
-    }
+                      VkFormatFeatureFlags2KHR buf_ff);
 
     void LinkChildNodes() override {
         // Connect child node(s), which cannot safely be done in the constructor.
@@ -118,4 +96,12 @@ class BUFFER_VIEW_STATE : public BASE_NODE {
         BASE_NODE::Destroy();
     }
     bool Invalid() const override { return Destroyed() || !buffer_state || buffer_state->Invalid(); }
+
+    VkDeviceSize Size() const {
+        VkDeviceSize size = create_info.range;
+        if (size == VK_WHOLE_SIZE) {
+            size = buffer_state->createInfo.size - create_info.offset;
+        }
+        return size;
+    }
 };
