@@ -46,8 +46,8 @@ struct DedicatedBinding {
 class DEVICE_MEMORY_STATE : public BASE_NODE {
   public:
     const safe_VkMemoryAllocateInfo alloc_info;
-    const VkExternalMemoryHandleTypeFlags export_handle_type_flags;
-    const VkExternalMemoryHandleTypeFlags import_handle_type_flags;
+    const VkExternalMemoryHandleTypeFlags export_handle_types;  // from VkExportMemoryAllocateInfo::handleTypes
+    const std::optional<VkExternalMemoryHandleTypeFlagBits> import_handle_type;
     const bool unprotected;     // can't be used for protected memory
     const bool multi_instance;  // Allocated from MULTI_INSTANCE heap or having more than one deviceMask bit set
     const std::optional<DedicatedBinding> dedicated;
@@ -63,11 +63,11 @@ class DEVICE_MEMORY_STATE : public BASE_NODE {
                         const VkMemoryType &memory_type, const VkMemoryHeap &memory_heap,
                         std::optional<DedicatedBinding> &&dedicated_binding, uint32_t physical_device_count);
 
-    bool IsImport() const { return import_handle_type_flags != 0; }
+    bool IsImport() const { return import_handle_type.has_value(); }
     bool IsImportAHB() const {
-        return (import_handle_type_flags & VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID) != 0;
+        return IsImport() && import_handle_type == VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID;
     }
-    bool IsExport() const { return export_handle_type_flags != 0; }
+    bool IsExport() const { return export_handle_types != 0; }
 
     bool IsDedicatedBuffer() const { return dedicated && dedicated->handle.type == kVulkanObjectTypeBuffer; }
 
@@ -322,8 +322,8 @@ class BindableMultiplanarMemoryTracker : public BindableMemoryTracker {
 class BINDABLE : public BASE_NODE {
   public:
     template <typename Handle>
-    BINDABLE(Handle h, VulkanObjectType t, bool is_sparse, bool is_unprotected, VkExternalMemoryHandleTypeFlags handle_type)
-        : BASE_NODE(h, t), external_memory_handle(handle_type), sparse(is_sparse), unprotected(is_unprotected) {}
+    BINDABLE(Handle h, VulkanObjectType t, bool is_sparse, bool is_unprotected, VkExternalMemoryHandleTypeFlags handle_types)
+        : BASE_NODE(h, t), external_memory_handle_types(handle_types), sparse(is_sparse), unprotected(is_unprotected) {}
 
     virtual ~BINDABLE() {
         if (!Destroyed()) {
@@ -334,7 +334,7 @@ class BINDABLE : public BASE_NODE {
     void Destroy() override { BASE_NODE::Destroy(); }
 
     bool IsExternalAHB() const {
-        return (external_memory_handle & VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID) != 0;
+        return (external_memory_handle_types & VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID) != 0;
     }
 
     const DEVICE_MEMORY_STATE *MemState() const {
@@ -373,9 +373,13 @@ class BINDABLE : public BASE_NODE {
     virtual void BindMemory(BASE_NODE *parent, std::shared_ptr<DEVICE_MEMORY_STATE> &mem, const VkDeviceSize memory_offset,
                             const VkDeviceSize resource_offset, const VkDeviceSize mem_size) = 0;
     virtual bool HasFullRangeBound() const = 0;
-    virtual bool DoesResourceMemoryOverlap(const sparse_container::range<VkDeviceSize> &memory_region,
-                                           const BINDABLE *other_resource,
-                                           const sparse_container::range<VkDeviceSize> &other_memory_region) const = 0;
+    virtual std::pair<VkDeviceMemory, sparse_container::range<VkDeviceSize>> GetResourceMemoryOverlap(
+        const sparse_container::range<VkDeviceSize> &memory_region, const BINDABLE *other_resource,
+        const sparse_container::range<VkDeviceSize> &other_memory_region) const = 0;
+    bool DoesResourceMemoryOverlap(const sparse_container::range<VkDeviceSize> &memory_region, const BINDABLE *other_resource,
+                                   const sparse_container::range<VkDeviceSize> &other_memory_region) const {
+        return GetResourceMemoryOverlap(memory_region, other_resource, other_memory_region).first != VK_NULL_HANDLE;
+    }
     virtual BindableMemoryTracker::BoundMemoryRange GetBoundMemoryRange(
         const sparse_container::range<VkDeviceSize> &range) const = 0;
     virtual BindableMemoryTracker::DeviceMemoryState GetBoundMemoryStates() const = 0;
@@ -390,7 +394,7 @@ class BINDABLE : public BASE_NODE {
 
   public:
     // Tracks external memory types creating resource
-    const VkExternalMemoryHandleTypeFlags external_memory_handle;
+    const VkExternalMemoryHandleTypeFlags external_memory_handle_types;
     const bool sparse;       // Is this object being bound with sparse memory or not?
     const bool unprotected;  // can't be used for protected memory
   protected:
@@ -423,27 +427,31 @@ class MEMORY_TRACKED_RESOURCE_STATE : public BaseClass {
 
     bool HasFullRangeBound() const override { return memory_tracker_.HasFullRangeBound(); }
 
-    bool DoesResourceMemoryOverlap(const sparse_container::range<VkDeviceSize> &memory_region, const BINDABLE *other_resource,
-                                   const sparse_container::range<VkDeviceSize> &other_memory_region) const override {
-        if (!other_resource) return false;
+    std::pair<VkDeviceMemory, sparse_container::range<VkDeviceSize>> GetResourceMemoryOverlap(
+        const sparse_container::range<VkDeviceSize> &memory_region, const BINDABLE *other_resource,
+        const sparse_container::range<VkDeviceSize> &other_memory_region) const override {
+        if (!other_resource) return {VK_NULL_HANDLE, {}};
 
         auto ranges = GetBoundMemoryRange(memory_region);
         auto other_ranges = other_resource->GetBoundMemoryRange(other_memory_region);
 
-        for (const auto &value_pair : ranges) {
+        for (const auto &[memory, memory_ranges] : ranges) {
             // Check if we have memory from same VkDeviceMemory bound
-            auto it = other_ranges.find(value_pair.first);
+            auto it = other_ranges.find(memory);
             if (it != other_ranges.end()) {
                 // Check if any of the bound memory ranges overlap
-                for (const auto &memory_range : value_pair.second) {
+                for (const auto &memory_range : memory_ranges) {
                     for (const auto &other_memory_range : it->second) {
-                        if (other_memory_range.intersects(memory_range)) return true;
+                        if (other_memory_range.intersects(memory_range)) {
+                            auto memory_space_intersection = other_memory_range & memory_range;
+                            return {memory, memory_space_intersection};
+                        }
                     }
                 }
             }
         }
 
-        return false;
+        return {VK_NULL_HANDLE, {}};
     }
 
     BindableMemoryTracker::BoundMemoryRange GetBoundMemoryRange(const sparse_container::range<VkDeviceSize> &range) const override {
