@@ -27,6 +27,7 @@
 #include <bitset>
 #include <iomanip>
 #include "cast_utils.h"
+#include "generated/vk_extension_helper.h"
 #include "generated/vk_format_utils.h"
 #include "error_message/logging.h"
 
@@ -63,13 +64,13 @@ static inline VkOffset3D CastTo3D(const VkOffset2D &d2) {
 }
 
 // Convert integer API version to a string
-static inline std::string StringAPIVersion(uint32_t version) {
+static inline std::string StringAPIVersion(APIVersion version) {
     std::stringstream version_name;
-    uint32_t major = VK_VERSION_MAJOR(version);
-    uint32_t minor = VK_VERSION_MINOR(version);
-    uint32_t patch = VK_VERSION_PATCH(version);
-    version_name << major << "." << minor << "." << patch << " (0x" << std::setfill('0') << std::setw(8) << std::hex << version
-                 << ")";
+    if (!version.valid()) {
+        return "<unrecognized>";
+    }
+    version_name << version.major() << "." << version.minor() << "." << version.patch() << " (0x" << std::setfill('0')
+                 << std::setw(8) << std::hex << version.value() << ")";
     return version_name.str();
 }
 
@@ -216,6 +217,29 @@ constexpr T Align(T x, T p2) {
 // Returns the 0-based index of the LSB. An input mask of 0 yields -1
 static inline int LeastSignificantBit(uint32_t mask) { return u_ffs(static_cast<int>(mask)) - 1; }
 
+// Compute a binomial coefficient
+template <typename T>
+constexpr T binom(T n, T k) {
+    static_assert(std::numeric_limits<T>::is_integer, "Unsigned integer required.");
+    static_assert(std::is_unsigned<T>::value, "Unsigned integer required.");
+    assert(n >= k);
+    if (n == 0) {
+        return 0;
+    }
+    if (k == 0) {
+        return 1;
+    }
+
+    T numerator = 1;
+    T denominator = 1;
+    for (T i = 1; i <= k; ++i) {
+        numerator *= n - i + 1;
+        denominator *= i;
+    }
+
+    return numerator / denominator;
+}
+
 template <typename FlagBits, typename Flags>
 FlagBits LeastSignificantFlag(Flags flags) {
     const int bit_shift = LeastSignificantBit(flags);
@@ -282,6 +306,12 @@ static inline bool IsImageLayoutReadOnly(VkImageLayout layout) {
                        [layout](const VkImageLayout read_only_layout) { return layout == read_only_layout; });
 }
 
+static inline bool IsImageLayoutDepthOnly(VkImageLayout layout) {
+    constexpr std::array depth_only_layouts = {VK_IMAGE_LAYOUT_DEPTH_ATTACHMENT_OPTIMAL, VK_IMAGE_LAYOUT_DEPTH_READ_ONLY_OPTIMAL};
+    return std::any_of(depth_only_layouts.begin(), depth_only_layouts.end(),
+                       [layout](const VkImageLayout read_only_layout) { return layout == read_only_layout; });
+}
+
 static inline bool IsImageLayoutDepthReadOnly(VkImageLayout layout) {
     constexpr std::array read_only_layouts = {
         VK_IMAGE_LAYOUT_DEPTH_STENCIL_READ_ONLY_OPTIMAL,
@@ -293,6 +323,13 @@ static inline bool IsImageLayoutDepthReadOnly(VkImageLayout layout) {
                        [layout](const VkImageLayout read_only_layout) { return layout == read_only_layout; });
 }
 
+static inline bool IsImageLayoutStencilOnly(VkImageLayout layout) {
+    constexpr std::array depth_only_layouts = {VK_IMAGE_LAYOUT_STENCIL_ATTACHMENT_OPTIMAL,
+                                               VK_IMAGE_LAYOUT_STENCIL_READ_ONLY_OPTIMAL};
+    return std::any_of(depth_only_layouts.begin(), depth_only_layouts.end(),
+                       [layout](const VkImageLayout read_only_layout) { return layout == read_only_layout; });
+}
+
 static inline bool IsImageLayoutStencilReadOnly(VkImageLayout layout) {
     constexpr std::array read_only_layouts = {
         VK_IMAGE_LAYOUT_DEPTH_STENCIL_READ_ONLY_OPTIMAL,
@@ -351,6 +388,33 @@ static inline uint32_t GetPlaneIndex(VkImageAspectFlags aspect) {
     }
 }
 
+// vkspec.html#formats-planes-image-aspect
+static inline bool IsValidPlaneAspect(VkFormat format, VkImageAspectFlags aspect_mask) {
+    const uint32_t planes = FormatPlaneCount(format);
+    constexpr VkImageAspectFlags valid_planes =
+        VK_IMAGE_ASPECT_PLANE_0_BIT | VK_IMAGE_ASPECT_PLANE_1_BIT | VK_IMAGE_ASPECT_PLANE_2_BIT;
+
+    if (((aspect_mask & valid_planes) == aspect_mask) && (aspect_mask != 0)) {
+        if ((planes == 3) || ((planes == 2) && ((aspect_mask & VK_IMAGE_ASPECT_PLANE_2_BIT) == 0))) {
+            return true;
+        }
+    }
+    return false;  // Expects calls to make sure it is a multi-planar format
+}
+
+static inline bool IsOnlyOneValidPlaneAspect(VkFormat format, VkImageAspectFlags aspect_mask) {
+    const bool multiple_bits = aspect_mask != 0 && !IsPowerOfTwo(aspect_mask);
+    return !multiple_bits && IsValidPlaneAspect(format, aspect_mask);
+}
+
+static inline bool IsMultiplePlaneAspect(VkImageAspectFlags aspect_mask) {
+    // If checking for multiple planes, there will already be another check if valid for plane count
+    constexpr VkImageAspectFlags valid_planes =
+        VK_IMAGE_ASPECT_PLANE_0_BIT | VK_IMAGE_ASPECT_PLANE_1_BIT | VK_IMAGE_ASPECT_PLANE_2_BIT;
+    const VkImageAspectFlags planes = aspect_mask & valid_planes;
+    return planes != 0 && !IsPowerOfTwo(planes);
+}
+
 // all "advanced blend operation" found in spec
 static inline bool IsAdvanceBlendOperation(const VkBlendOp blend_op) {
     return (static_cast<int>(blend_op) >= VK_BLEND_OP_ZERO_EXT) && (static_cast<int>(blend_op) <= VK_BLEND_OP_BLUE_EXT);
@@ -371,6 +435,8 @@ static inline bool IsBetweenInclusive(const VkExtent2D &value, const VkExtent2D
     return IsBetweenInclusive(value.width, min.width, max.width) && IsBetweenInclusive(value.height, min.height, max.height);
 }
 
+static inline bool IsBetweenInclusive(float value, float min, float max) { return (value >= min) && (value <= max); }
+
 // Check if value is integer multiple of granularity
 static inline bool IsIntegerMultipleOf(VkDeviceSize value, VkDeviceSize granularity) {
     if (granularity == 0) {
@@ -425,8 +491,8 @@ static inline uint32_t FullMipChainLevels(VkExtent3D extent) {
 }
 
 // Returns the effective extent of an image subresource, adjusted for mip level and array depth.
-[[nodiscard]] constexpr VkExtent3D GetEffectiveExtent(const VkImageCreateInfo &ci, const VkImageAspectFlags aspect_mask,
-                                                      const uint32_t mip_level) {
+[[nodiscard]] inline VkExtent3D GetEffectiveExtent(const VkImageCreateInfo &ci, const VkImageAspectFlags aspect_mask,
+                                                   const uint32_t mip_level) {
     // Return zero extent if mip level doesn't exist
     if (mip_level >= ci.mipLevels) {
         return VkExtent3D{0, 0, 0};
@@ -466,7 +532,7 @@ static inline uint32_t FullMipChainLevels(VkExtent3D extent) {
 }
 
 // Returns the effective extent of an image subresource, adjusted for mip level and array depth.
-[[nodiscard]] constexpr VkExtent3D GetEffectiveExtent(const VkImageCreateInfo &ci, const VkImageSubresourceRange &range) {
+[[nodiscard]] inline VkExtent3D GetEffectiveExtent(const VkImageCreateInfo &ci, const VkImageSubresourceRange &range) {
     return GetEffectiveExtent(ci, range.aspectMask, range.baseMipLevel);
 }
 
@@ -482,9 +548,9 @@ constexpr uint32_t ResolveRemainingLayers(const VkImageCreateInfo &ci, VkImageSu
 
 // Find whether or not an element is in list
 // Two definitions, to be able to do the following calls:
-// IsIn(1, {1, 2, 3});
+// IsValueIn(1, {1, 2, 3});
 // std::array arr {1, 2, 3};
-// IsIn(1, arr);
+// IsValueIn(1, arr);
 template <typename T, typename RANGE>
 bool IsValueIn(const T &v, const RANGE &range) {
     return std::find(std::begin(range), std::end(range), v) != std::end(range);
@@ -709,4 +775,41 @@ class vl_concurrent_unordered_map {
         return hash;
     }
 };
+
+static constexpr VkPipelineStageFlags2KHR kFramebufferStagePipelineStageFlags =
+    (VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT | VK_PIPELINE_STAGE_EARLY_FRAGMENT_TESTS_BIT |
+     VK_PIPELINE_STAGE_LATE_FRAGMENT_TESTS_BIT | VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT);
+
+static constexpr VkAccessFlags2 kShaderTileImageAllowedAccessFlags =
+    VK_ACCESS_2_COLOR_ATTACHMENT_READ_BIT | VK_ACCESS_2_COLOR_ATTACHMENT_WRITE_BIT | VK_ACCESS_2_DEPTH_STENCIL_ATTACHMENT_READ_BIT |
+    VK_ACCESS_2_DEPTH_STENCIL_ATTACHMENT_WRITE_BIT;
+
+static constexpr bool HasNonFramebufferStagePipelineStageFlags(VkPipelineStageFlags2KHR inflags) {
+    return (inflags & ~kFramebufferStagePipelineStageFlags) != 0;
+}
+
+static constexpr bool HasFramebufferStagePipelineStageFlags(VkPipelineStageFlags2KHR inflags) {
+    return (inflags & kFramebufferStagePipelineStageFlags) != 0;
+}
+
+static constexpr bool HasNonShaderTileImageAccessFlags(VkAccessFlags2 in_flags) {
+    return ((in_flags & ~kShaderTileImageAllowedAccessFlags) != 0);
+}
+
+namespace vvl {
+
+static inline void ToLower(std::string &str) {
+    // std::tolower() returns int which can cause compiler warnings
+    transform(str.begin(), str.end(), str.begin(),
+              [](char c) { return static_cast<char>(std::tolower(c)); });
+}
+
+static inline void ToUpper(std::string &str) {
+    // std::toupper() returns int which can cause compiler warnings
+    transform(str.begin(), str.end(), str.begin(),
+              [](char c) { return static_cast<char>(std::toupper(c)); });
+}
+
+}  // namespace vvl
+
 #endif
