@@ -24,7 +24,8 @@
 #include <memory>
 #include <vector>
 
-#include "generated/lvt_function_pointers.h"
+#include "generated/vk_function_pointers.h"
+#include "generated/vk_extension_helper.h"
 #include "test_common.h"
 
 namespace vk_testing {
@@ -62,7 +63,6 @@ class BufferView;
 class Image;
 class ImageView;
 class DepthStencilView;
-class Shader;
 class Pipeline;
 class PipelineDelta;
 class Sampler;
@@ -336,12 +336,6 @@ class DeviceMemory : public internal::NonDispHandle<VkDeviceMemory> {
 
 class Fence : public internal::NonDispHandle<VkFence> {
   public:
-#ifdef _WIN32
-    using ExternalHandle = HANDLE;
-#else
-    using ExternalHandle = int;
-#endif
-
     Fence() = default;
     Fence(const Device &dev) { init(dev, create_info()); }
     Fence(const Device &dev, const VkFenceCreateInfo &info) { init(dev, info); }
@@ -357,8 +351,12 @@ class Fence : public internal::NonDispHandle<VkFence> {
 
     VkResult reset();
 
-    VkResult export_handle(ExternalHandle &handle, VkExternalFenceHandleTypeFlagBits handle_type);
-    VkResult import_handle(ExternalHandle handle, VkExternalFenceHandleTypeFlagBits handle_type, VkFenceImportFlags flags = 0);
+#ifdef VK_USE_PLATFORM_WIN32_KHR
+    VkResult export_handle(HANDLE &win32_handle, VkExternalFenceHandleTypeFlagBits handle_type);
+    VkResult import_handle(HANDLE win32_handle, VkExternalFenceHandleTypeFlagBits handle_type, VkFenceImportFlags flags = 0);
+#endif
+    VkResult export_handle(int &fd_handle, VkExternalFenceHandleTypeFlagBits handle_type);
+    VkResult import_handle(int fd_handle, VkExternalFenceHandleTypeFlagBits handle_type, VkFenceImportFlags flags = 0);
 
     static VkFenceCreateInfo create_info(VkFenceCreateFlags flags);
     static VkFenceCreateInfo create_info();
@@ -366,12 +364,6 @@ class Fence : public internal::NonDispHandle<VkFence> {
 
 class Semaphore : public internal::NonDispHandle<VkSemaphore> {
   public:
-#ifdef _WIN32
-    using ExternalHandle = HANDLE;
-#else
-    using ExternalHandle = int;
-#endif
-
     Semaphore() = default;
     Semaphore(const Device &dev) { init(dev, LvlInitStruct<VkSemaphoreCreateInfo>()); }
     Semaphore(const Device &dev, const VkSemaphoreCreateInfo &info) { init(dev, info); }
@@ -381,9 +373,13 @@ class Semaphore : public internal::NonDispHandle<VkSemaphore> {
     // vkCreateSemaphore()
     void init(const Device &dev, const VkSemaphoreCreateInfo &info);
 
-    VkResult export_handle(ExternalHandle &ext_handle, VkExternalSemaphoreHandleTypeFlagBits handle_type);
-    VkResult import_handle(ExternalHandle ext_handle, VkExternalSemaphoreHandleTypeFlagBits handle_type,
+#ifdef VK_USE_PLATFORM_WIN32_KHR
+    VkResult export_handle(HANDLE &win32_handle, VkExternalSemaphoreHandleTypeFlagBits handle_type);
+    VkResult import_handle(HANDLE win32_handle, VkExternalSemaphoreHandleTypeFlagBits handle_type,
                            VkSemaphoreImportFlags flags = 0);
+#endif
+    VkResult export_handle(int &fd_handle, VkExternalSemaphoreHandleTypeFlagBits handle_type);
+    VkResult import_handle(int fd_handle, VkExternalSemaphoreHandleTypeFlagBits handle_type, VkSemaphoreImportFlags flags = 0);
 
     static VkSemaphoreCreateInfo create_info(VkFlags flags);
 };
@@ -436,25 +432,15 @@ static constexpr NoMemT no_mem{};
 class Buffer : public internal::NonDispHandle<VkBuffer> {
   public:
     explicit Buffer() : NonDispHandle(), create_info_(LvlInitStruct<decltype(create_info_)>()) {}
-    explicit Buffer(const Device &dev, const VkBufferCreateInfo &info) { init(dev, info); }
-    explicit Buffer(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props) {
-        init(dev, info, mem_props);
-    }
-    explicit Buffer(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props, void *alloc_info_pnext) {
+    explicit Buffer(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props = 0,
+                    void *alloc_info_pnext = nullptr) {
         init(dev, info, mem_props, alloc_info_pnext);
     }
-    explicit Buffer(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props,
-                    VkMemoryAllocateFlags alloc_flags) {
-        auto memflagsinfo = LvlInitStruct<VkMemoryAllocateFlagsInfo>();
-        memflagsinfo.flags = alloc_flags;
-        init(dev, info, mem_props, &memflagsinfo);
-    }
-    explicit Buffer(const Device &dev, VkDeviceSize size, VkMemoryPropertyFlags mem_props, VkBufferUsageFlags usage,
-                    void *alloc_info_pnext) {
-        init(dev, size, mem_props, usage, alloc_info_pnext);
+    explicit Buffer(const Device &dev, VkDeviceSize size, VkBufferUsageFlags usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT, VkMemoryPropertyFlags mem_props = 0,
+                    void *alloc_info_pnext = nullptr) {
+        init(dev, size, usage, mem_props, alloc_info_pnext);
     }
     explicit Buffer(const Device &dev, const VkBufferCreateInfo &info, NoMemT) { init_no_mem(dev, info); }
-    explicit Buffer(const Device &dev, VkDeviceSize size) { init(dev, size); }
     Buffer(Buffer &&rhs) noexcept : NonDispHandle(std::move(rhs)) {
         create_info_ = std::move(rhs.create_info_);
         internal_mem_ = std::move(rhs.internal_mem_);
@@ -474,38 +460,13 @@ class Buffer : public internal::NonDispHandle<VkBuffer> {
     void destroy() noexcept;
 
     // vkCreateBuffer()
-    void init(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props, void *alloc_info_pnext = nullptr);
-    void init(const Device &dev, const VkBufferCreateInfo &info) { init(dev, info, 0); }
-    void init(const Device &dev, VkDeviceSize size, VkMemoryPropertyFlags mem_props,
-              VkBufferUsageFlags usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT, const std::vector<uint32_t> &queue_families = {}) {
-        init(dev, create_info(size, usage, &queue_families), mem_props);
+    void init(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props = 0,
+              void *alloc_info_pnext = nullptr);
+    void init(const Device &dev, VkDeviceSize size, VkBufferUsageFlags usage = VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT,
+              VkMemoryPropertyFlags mem_props = 0, void *alloc_info_pnext = nullptr,
+              const std::vector<uint32_t> &queue_families = {}) {
+        init(dev, create_info(size, usage, &queue_families), mem_props, alloc_info_pnext);
     }
-    void init(const Device &dev, VkDeviceSize size, VkMemoryPropertyFlags mem_props, VkBufferUsageFlags usage,
-              void *alloc_info_pnext) {
-        init(dev, create_info(size, usage), mem_props, alloc_info_pnext);
-    }
-    void init(const Device &dev, VkDeviceSize size) { init(dev, size, 0); }
-    void init_as_src(const Device &dev, VkDeviceSize size, VkMemoryPropertyFlags &reqs,
-                     const std::vector<uint32_t> *queue_families = nullptr) {
-        init(dev, create_info(size, VK_BUFFER_USAGE_TRANSFER_SRC_BIT, queue_families), reqs);
-    }
-    void init_as_dst(const Device &dev, VkDeviceSize size, VkMemoryPropertyFlags &reqs,
-                     const std::vector<uint32_t> *queue_families = nullptr) {
-        init(dev, create_info(size, VK_BUFFER_USAGE_TRANSFER_DST_BIT, queue_families), reqs);
-    }
-    void init_as_src_and_dst(const Device &dev, VkDeviceSize size, VkMemoryPropertyFlags &reqs,
-                             const std::vector<uint32_t> *queue_families = nullptr, bool memory = true) {
-        if (memory)
-            init(dev, create_info(size, VK_BUFFER_USAGE_TRANSFER_SRC_BIT | VK_BUFFER_USAGE_TRANSFER_DST_BIT, queue_families), reqs);
-        else
-            init_no_mem(dev,
-                        create_info(size, VK_BUFFER_USAGE_TRANSFER_SRC_BIT | VK_BUFFER_USAGE_TRANSFER_DST_BIT, queue_families));
-    }
-    void init_as_storage(const Device &dev, VkDeviceSize size, VkMemoryPropertyFlags &reqs,
-        const std::vector<uint32_t> *queue_families = nullptr) {
-        init(dev, create_info(size, VK_BUFFER_USAGE_STORAGE_BUFFER_BIT, queue_families), reqs);
-    }
-
     void init_no_mem(const Device &dev, const VkBufferCreateInfo &info);
 
     // get the internal memory
@@ -515,10 +476,12 @@ class Buffer : public internal::NonDispHandle<VkBuffer> {
     // vkGetObjectMemoryRequirements()
     VkMemoryRequirements memory_requirements() const;
 
-    // vkBindObjectMemory()
+    // Allocate and bind memory
+    // The assumption that this object was created in no_mem configuration
+    void allocate_and_bind_memory(const Device &dev, VkMemoryPropertyFlags mem_props = 0, void *alloc_info_pnext = nullptr);
+
+    // Bind to existing memory object
     void bind_memory(const DeviceMemory &mem, VkDeviceSize mem_offset);
-    // Bind to internal_mem_ reference
-    void bind_memory(const Device &dev, VkMemoryPropertyFlags mem_props, VkDeviceSize mem_offset);
 
     const VkBufferCreateInfo &create_info() const { return create_info_; }
     static VkBufferCreateInfo create_info(VkDeviceSize size, VkFlags usage, const std::vector<uint32_t> *queue_families = nullptr,
@@ -557,7 +520,7 @@ class Buffer : public internal::NonDispHandle<VkBuffer> {
         return barrier;
     }
 
-    [[nodiscard]] VkDeviceAddress address(uint32_t vk_api_version = VK_API_VERSION_1_2) const;
+    [[nodiscard]] VkDeviceAddress address(APIVersion vk_api_version = VK_API_VERSION_1_2) const;
 
   private:
     VkBufferCreateInfo create_info_;
@@ -610,7 +573,11 @@ class Image : public internal::NonDispHandle<VkImage> {
     // vkGetObjectMemoryRequirements()
     VkMemoryRequirements memory_requirements() const;
 
-    // vkBindObjectMemory()
+    // Allocate and bind memory
+    // The assumption that this object was created in no_mem configuration
+    void allocate_and_bind_memory(const Device &dev, VkMemoryPropertyFlags mem_props = 0, void *alloc_info_pnext = nullptr);
+
+    // Bind to existing memory object
     void bind_memory(const DeviceMemory &mem, VkDeviceSize mem_offset);
 
     // vkGetImageSubresourceLayout()
@@ -745,6 +712,23 @@ class ShaderModule : public internal::NonDispHandle<VkShaderModule> {
     static VkShaderModuleCreateInfo create_info(size_t code_size, const uint32_t *code, VkFlags flags);
 };
 
+class Shader : public internal::NonDispHandle<VkShaderEXT> {
+  public:
+    Shader(const Device &dev, const VkShaderCreateInfoEXT &info) { init(dev, info); }
+    Shader(const Device &dev, const VkShaderStageFlagBits stage, const std::vector<uint32_t> &spv,
+           const VkDescriptorSetLayout *descriptorSetLayout = nullptr, const VkPushConstantRange* pushConstRange = nullptr);
+    Shader(const Device &dev, const VkShaderStageFlagBits stage, const std::vector<uint8_t> &binary,
+           const VkDescriptorSetLayout *descriptorSetLayout = nullptr, const VkPushConstantRange *pushConstRange = nullptr);
+    Shader(const Device &dev, const VkShaderStageFlagBits stage, const std::vector<uint32_t> &spv,
+           VkShaderCreateFlagsEXT flags);
+    ~Shader() noexcept;
+    void destroy() noexcept;
+
+    // vkCreateShaderModule()
+    void init(const Device &dev, const VkShaderCreateInfoEXT &info);
+    VkResult init_try(const Device &dev, const VkShaderCreateInfoEXT &info);
+};
+
 class Pipeline : public internal::NonDispHandle<VkPipeline> {
   public:
     Pipeline() = default;
@@ -939,7 +923,9 @@ class CommandBuffer : public internal::Handle<VkCommandBuffer> {
 class RenderPass : public internal::NonDispHandle<VkRenderPass> {
   public:
     RenderPass() = default;
+    // vkCreateRenderPass()
     RenderPass(const Device &dev, const VkRenderPassCreateInfo &info) { init(dev, info); }
+    // vkCreateRenderPass2()
     RenderPass(const Device &dev, const VkRenderPassCreateInfo2 &info, bool khr = false) { init(dev, info, khr); }
     ~RenderPass() noexcept;
     void destroy() noexcept;
@@ -1226,25 +1212,31 @@ struct GraphicsPipelineFromLibraries {
     VkPipelineLibraryCreateInfoKHR link_info;
     vk_testing::Pipeline pipe;
 
-    GraphicsPipelineFromLibraries(const Device &dev, vvl::span<VkPipeline> libs, VkGraphicsPipelineCreateInfo *ci = nullptr)
-        : libs(libs) {
-        link_info = LvlInitStruct<VkPipelineLibraryCreateInfoKHR>();
-        link_info.libraryCount = static_cast<uint32_t>(libs.size());
-        link_info.pLibraries = libs.data();
+    GraphicsPipelineFromLibraries(const Device &dev, vvl::span<VkPipeline> libs, VkPipelineLayout layout)
+        : GraphicsPipelineFromLibraries(libs) {
+        auto exe_pipe_ci = LvlInitStruct<VkGraphicsPipelineCreateInfo>(&link_info);
+        exe_pipe_ci.layout = layout;
+        pipe.init(dev, exe_pipe_ci);
+        pipe.initialized();
+    }
 
-        if (ci) {
-            link_info.pNext = ci->pNext;
-            ci->pNext = &link_info;
-            pipe.init(dev, *ci);
-        } else {
-            auto exe_pipe_ci = LvlInitStruct<VkGraphicsPipelineCreateInfo>(&link_info);
-            pipe.init(dev, exe_pipe_ci);
-        }
+    GraphicsPipelineFromLibraries(const Device &dev, vvl::span<VkPipeline> libs, VkGraphicsPipelineCreateInfo &ci)
+        : GraphicsPipelineFromLibraries(libs) {
+        link_info.pNext = ci.pNext;
+        ci.pNext = &link_info;
+        pipe.init(dev, ci);
         pipe.initialized();
     }
 
     operator VkPipeline() const { return pipe.handle(); }
     operator bool() const { return pipe.initialized(); }
+
+  private:
+    GraphicsPipelineFromLibraries(vvl::span<VkPipeline> libs) : libs(libs) {
+        link_info = LvlInitStruct<VkPipelineLibraryCreateInfoKHR>();
+        link_info.libraryCount = static_cast<uint32_t>(libs.size());
+        link_info.pLibraries = libs.data();
+    }
 };
 
 }  // namespace vk_testing
