@@ -18,22 +18,19 @@
 
 #pragma once
 
-#include "generated/lvt_function_pointers.h"
+#include "generated/vk_function_pointers.h"
 #include "error_monitor.h"
 
-#ifdef ANDROID
-#include "test_framework_android.h"
-class VkImageObj;
-#else
+#if defined(VVL_TESTS_USE_CMAKE)
 #include "test_framework.h"
+#else
+#include "test_framework_android.h"
 #endif
 
-#if defined(ANDROID)
+#if defined(VK_USE_PLATFORM_ANDROID_KHR)
 #include <android/log.h>
-#if defined(VALIDATION_APK)
 #include <android_native_app_glue.h>
 #endif
-#endif
 
 #include <algorithm>
 #include <array>
@@ -80,50 +77,8 @@ class VkDeviceObj : public vk_testing::Device {
     VkQueueObj *m_queue_obj = nullptr;
     VkQueue m_queue;
 };
-struct DebugReporter {
-    void Create(VkInstance instance) noexcept;
-    void Destroy(VkInstance instance) noexcept;
-
-    ErrorMonitor error_monitor_;
-
-#ifdef VK_USE_PLATFORM_ANDROID_KHR
-    static VKAPI_ATTR VkBool32 VKAPI_CALL DebugCallback(VkDebugReportFlagsEXT message_flags, VkDebugReportObjectTypeEXT, uint64_t,
-                                                        size_t, int32_t, const char *, const char *msg, void *user_data);
-
-    const char *debug_extension_name = VK_EXT_DEBUG_REPORT_EXTENSION_NAME;
-    VkDebugReportCallbackCreateInfoEXT debug_create_info_ = {VK_STRUCTURE_TYPE_DEBUG_REPORT_CALLBACK_CREATE_INFO_EXT, nullptr,
-                                                             VK_DEBUG_REPORT_ERROR_BIT_EXT | VK_DEBUG_REPORT_WARNING_BIT_EXT |
-                                                                 VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT |
-                                                                 VK_DEBUG_REPORT_INFORMATION_BIT_EXT,
-                                                             &DebugCallback, &error_monitor_};
-    using DebugCreateFnType = PFN_vkCreateDebugReportCallbackEXT;
-    const char *debug_create_fn_name_ = "vkCreateDebugReportCallbackEXT";
-    using DebugDestroyFnType = PFN_vkDestroyDebugReportCallbackEXT;
-    const char *debug_destroy_fn_name_ = "vkDestroyDebugReportCallbackEXT";
-    VkDebugReportCallbackEXT debug_obj_ = VK_NULL_HANDLE;
-#else
-    static VKAPI_ATTR VkBool32 VKAPI_CALL DebugCallback(VkDebugUtilsMessageSeverityFlagBitsEXT message_severity,
-                                                        VkDebugUtilsMessageTypeFlagsEXT message_types,
-                                                        const VkDebugUtilsMessengerCallbackDataEXT *callback_data, void *user_data);
-
-    const char *debug_extension_name = VK_EXT_DEBUG_UTILS_EXTENSION_NAME;
-    VkDebugUtilsMessengerCreateInfoEXT debug_create_info_ = {
-        VK_STRUCTURE_TYPE_DEBUG_UTILS_MESSENGER_CREATE_INFO_EXT,
-        nullptr,
-        0,
-        VK_DEBUG_UTILS_MESSAGE_SEVERITY_ERROR_BIT_EXT | VK_DEBUG_UTILS_MESSAGE_SEVERITY_WARNING_BIT_EXT |
-            VK_DEBUG_UTILS_MESSAGE_SEVERITY_INFO_BIT_EXT,
-        VK_DEBUG_UTILS_MESSAGE_TYPE_VALIDATION_BIT_EXT | VK_DEBUG_UTILS_MESSAGE_TYPE_PERFORMANCE_BIT_EXT,
-        &DebugCallback,
-        &error_monitor_};
-    using DebugCreateFnType = PFN_vkCreateDebugUtilsMessengerEXT;
-    const char *debug_create_fn_name_ = "vkCreateDebugUtilsMessengerEXT";
-    using DebugDestroyFnType = PFN_vkDestroyDebugUtilsMessengerEXT;
-    const char *debug_destroy_fn_name_ = "vkDestroyDebugUtilsMessengerEXT";
-    VkDebugUtilsMessengerEXT debug_obj_ = VK_NULL_HANDLE;
-#endif
-};
 
+class VkImageObj;
 class VkCommandPoolObj;
 class VkCommandBufferObj;
 class VkDepthStencilObj;
@@ -131,24 +86,14 @@ class VkDepthStencilObj;
 typedef enum {
     kGalaxyS10,
     kPixel3,
-    kPixelC,
-    kNexusPlayer,
-    kShieldTV,
     kShieldTVb,
-    kPixel3aXL,
-    kPixel2XL,
     kMockICD,
 } PlatformType;
 
 const std::unordered_map<PlatformType, std::string, std::hash<int>> vk_gpu_table = {
     {kGalaxyS10, "Mali-G76"},
     {kPixel3, "Adreno (TM) 630"},
-    {kPixelC, "NVIDIA Tegra X1"},
-    {kNexusPlayer, "PowerVR Rogue G6430"},
-    {kShieldTV, "NVIDIA Tegra X1 (nvgpu)"},
     {kShieldTVb, "NVIDIA Tegra X1 (rev B) (nvgpu)"},
-    {kPixel3aXL, "Adreno (TM) 615"},
-    {kPixel2XL, "Adreno (TM) 540"},
     {kMockICD, "Vulkan Mock Device"},
 };
 struct SurfaceContext {
@@ -212,6 +157,8 @@ class VkRenderFramework : public VkTestFramework {
     void InitRenderTarget(uint32_t targets);
     void InitRenderTarget(VkImageView *dsBinding);
     void InitRenderTarget(uint32_t targets, VkImageView *dsBinding);
+    void InitDynamicRenderTarget(VkFormat format = VK_FORMAT_UNDEFINED);
+    VkImageView GetDynamicRenderTarget() const;
     void DestroyRenderTarget();
 
     static bool IgnoreDisableChecks();
@@ -264,8 +211,8 @@ class VkRenderFramework : public VkTestFramework {
     std::vector<VkLayerProperties> available_layers_; // allow caching of available layers
     std::vector<VkExtensionProperties> available_extensions_; // allow caching of available instance extensions
 
-    DebugReporter debug_reporter_;
-    ErrorMonitor *m_errorMonitor = &debug_reporter_.error_monitor_;  // compatibility alias name
+    ErrorMonitor monitor_ = ErrorMonitor(m_print_vu);
+    ErrorMonitor *m_errorMonitor = &monitor_;  // TODO: Removing this properly is it's own PR. It's a big change.
 
     VkApplicationInfo app_info_;
     std::vector<const char *> instance_layers_;
@@ -300,16 +247,6 @@ class VkRenderFramework : public VkTestFramework {
 
     std::vector<VkViewport> m_viewports;
     std::vector<VkRect2D> m_scissors;
-    float m_lineWidth;
-    float m_depthBiasConstantFactor;
-    float m_depthBiasClamp;
-    float m_depthBiasSlopeFactor;
-    float m_blendConstants[4];
-    float m_minDepthBounds;
-    float m_maxDepthBounds;
-    uint32_t m_compareMask;
-    uint32_t m_writeMask;
-    uint32_t m_reference;
     bool m_addRenderPassSelfDependency;
     std::vector<VkSubpassDependency> m_additionalSubpassDependencies;
     std::vector<VkClearValue> m_renderPassClearValues;
@@ -318,6 +255,7 @@ class VkRenderFramework : public VkTestFramework {
     uint32_t m_width, m_height;
     VkFormat m_render_target_fmt;
     VkFormat m_depth_stencil_fmt;
+    VkImageLayout m_depth_stencil_layout;
     VkClearColorValue m_clear_color;
     bool m_clear_via_load_op;
     float m_depth_clear_color;
@@ -333,7 +271,7 @@ class VkRenderFramework : public VkTestFramework {
     // Device extensions to enable
     std::vector<const char *> m_device_extension_names;
 
-    VkValidationFeaturesEXT validation_features;
+    VkValidationFeaturesEXT m_validation_features;
     VkValidationFeatureEnableEXT validation_enable_all[4] = {VK_VALIDATION_FEATURE_ENABLE_GPU_ASSISTED_EXT,
                                                              VK_VALIDATION_FEATURE_ENABLE_BEST_PRACTICES_EXT,
                                                              VK_VALIDATION_FEATURE_ENABLE_GPU_ASSISTED_RESERVE_BINDING_SLOT_EXT,
@@ -397,6 +335,7 @@ class VkCommandBufferObj : public vk_testing::CommandBuffer {
     void NextSubpass(VkSubpassContents contents = VK_SUBPASS_CONTENTS_INLINE);
     void EndRenderPass();
     void BeginRendering(const VkRenderingInfoKHR &renderingInfo);
+    void BeginRenderingColor(const VkImageView imageView);
     void EndRendering();
     void BeginVideoCoding(const VkVideoBeginCodingInfoKHR &beginInfo);
     void ControlVideoCoding(const VkVideoCodingControlInfoKHR &controlInfo);
@@ -483,7 +422,7 @@ class VkImageObj : public vk_testing::Image {
 
     void Layout(VkImageLayout const layout) { m_descriptorImageInfo.imageLayout = layout; }
 
-    VkDeviceMemory memory() const { return Image::memory().handle(); }
+    VkDeviceMemory Memory() const { return Image::memory().handle(); }
 
     void *MapMemory() { return Image::memory().map(); }
 
@@ -503,21 +442,16 @@ class VkImageObj : public vk_testing::Image {
 
     VkImage image() const { return handle(); }
 
-    VkImageViewCreateInfo TargetViewCI(VkFormat format) const {
+    VkImageViewCreateInfo BasicViewCreatInfo(VkImageAspectFlags aspect_mask = VK_IMAGE_ASPECT_COLOR_BIT) const {
         auto ci = LvlInitStruct<VkImageViewCreateInfo>();
-        ci.format = format;
+        ci.image = handle();
+        ci.format = format();
         ci.viewType = VK_IMAGE_VIEW_TYPE_2D;
         ci.components.r = VK_COMPONENT_SWIZZLE_R;
         ci.components.g = VK_COMPONENT_SWIZZLE_G;
         ci.components.b = VK_COMPONENT_SWIZZLE_B;
         ci.components.a = VK_COMPONENT_SWIZZLE_A;
-        ci.subresourceRange = {
-            VK_IMAGE_ASPECT_COLOR_BIT,
-            0,                          // base mip level
-            VK_REMAINING_MIP_LEVELS,    // level count
-            0,                          // base array layer
-            VK_REMAINING_ARRAY_LAYERS,  // layer count
-        };
+        ci.subresourceRange = {aspect_mask, 0, 1, 0, 1};
         ci.flags = 0;
         return ci;
     }
@@ -661,7 +595,7 @@ class VkShaderObj : public vk_testing::ShaderModule {
     // optional arguments listed order of most likely to be changed manually by a test
     VkShaderObj(VkRenderFramework *framework, const char *source, VkShaderStageFlagBits stage,
                 const spv_target_env env = SPV_ENV_VULKAN_1_0, SpvSourceType source_type = SPV_SOURCE_GLSL,
-                const VkSpecializationInfo *spec_info = nullptr, char const *name = "main", bool debug = false);
+                const VkSpecializationInfo *spec_info = nullptr, char const *entry_point = "main", bool debug = false);
     VkPipelineShaderStageCreateInfo const &GetStageCreateInfo() const;
 
     bool InitFromGLSL(bool debug = false);
@@ -671,17 +605,16 @@ class VkShaderObj : public vk_testing::ShaderModule {
 
     // These functions return a pointer to a newly created _and initialized_ VkShaderObj if initialization was successful.
     // Otherwise, {} is returned.
-    static std::unique_ptr<VkShaderObj> CreateFromGLSL(VkRenderFramework &framework, VkShaderStageFlagBits stage,
-                                                       const std::string &code, const char *entry_point = "main",
+    static std::unique_ptr<VkShaderObj> CreateFromGLSL(VkRenderFramework *framework, const char *source,
+                                                       VkShaderStageFlagBits stage, const spv_target_env = SPV_ENV_VULKAN_1_0,
                                                        const VkSpecializationInfo *spec_info = nullptr,
-                                                       const spv_target_env = SPV_ENV_VULKAN_1_0, bool debug = false);
-    static std::unique_ptr<VkShaderObj> CreateFromASM(VkRenderFramework &framework, VkShaderStageFlagBits stage,
-                                                      const std::string &code, const char *entry_point = "main",
+                                                       const char *entry_point = "main", bool debug = false);
+    static std::unique_ptr<VkShaderObj> CreateFromASM(VkRenderFramework *framework, const char *source, VkShaderStageFlagBits stage,
+                                                      const spv_target_env spv_env = SPV_ENV_VULKAN_1_0,
                                                       const VkSpecializationInfo *spec_info = nullptr,
-                                                      const spv_target_env spv_env = SPV_ENV_VULKAN_1_0);
+                                                      const char *entry_point = "main");
 
-    // TODO (ncesario) remove ifndef once android build consolidation changes go in
-#ifndef __ANDROID__
+#if defined(VVL_TESTS_USE_CMAKE)
     struct GlslangTargetEnv {
         GlslangTargetEnv(const spv_target_env env) {
             switch (env) {
